@@ -9,7 +9,7 @@ import jmespath
 import pygments.lexers
 
 
-__version__ = '0.1.0'
+__version__ = '0.1.1'
 
 
 SAMPLE_JSON = {
@@ -63,6 +63,7 @@ class JMESPathDisplay(object):
         self.parsed_json = input_data
         self.lexer = pygments.lexers.get_lexer_by_name('json')
         self.formatter = ConsoleJSONFormatter()
+        self.saved_expressions = []
 
     def _create_colorized_json(self, json_string):
         tokens = self.lexer.get_tokens(json_string)
@@ -110,6 +111,12 @@ class JMESPathDisplay(object):
                                 footer=self.footer, focus_part='header')
 
     def _on_edit(self, widget, text):
+        if not text:
+            # If a user has hit backspace until there's no expression
+            # left, we can exit early and just clear the result text
+            # panel.
+            self.jmespath_result.set_text('')
+            return
         try:
             result = jmespath.compile(text).search(self.parsed_json)
             self.footer.set_text("Status: success")
@@ -132,12 +139,38 @@ class JMESPathDisplay(object):
     def unhandled_input(self, key):
         if key == 'f5':
             raise urwid.ExitMainLoop()
-        if key == 'ctrl ]':
+        elif key == 'ctrl ]':
             # Keystroke to quickly empty out the
             # currently entered expression.  Avoids
             # having to hold backspace to delete
             # the current expression current expression.
             self.input_expr.edit_text = ''
+            self.jmespath_result.set_text('')
+        elif key == 'ctrl p':
+            self.saved_expressions.append(self.input_expr.edit_text)
+            self.footer.set_text("Status: expression saved")
+
+    def display_saved_expressions(self):
+        for expression in self.saved_expressions:
+            print(expression)
+
+
+def _load_input_json(filename):
+    if filename is not None:
+        with open(filename) as f:
+            input_json = json.load(f)
+    elif not os.isatty(sys.stdin.fileno()):
+        # If stdin is a pipe, we need read the JSON from
+        # stdin and then reset stdin this back to the controlling tty.
+        input_json = json.loads(sys.stdin.read())
+        sys.stdin = open(os.ctermid(), 'r')
+    else:
+        # If the user didn't provide a filename,
+        # we want to be helpful so we'll use a sample
+        # document so they can still try out the
+        # JMESPath Terminal.
+        input_json = SAMPLE_JSON
+    return input_json
 
 
 def main():
@@ -150,23 +183,18 @@ def main():
                         version='jmespath-term %s' % __version__)
 
     args = parser.parse_args()
-    input_json = getattr(args, 'input-json', None)
-    if input_json is not None:
-        with open(input_json) as f:
-            input_json = json.load(f)
-    else:
-        input_json = SAMPLE_JSON
-
-    if not os.isatty(sys.stdin.fileno()):
-        # If stdin is a pipe, we need read the JSON from
-        #stdin and then reset stdin this back to the controlling tty.
-        input_json = json.loads(sys.stdin.read())
-        sys.stdin = open(os.ctermid(), 'r')
+    try:
+        input_json = _load_input_json(getattr(args, 'input-json', None))
+    except ValueError as e:
+        sys.stderr.write("Unable to load the input JSON: %s\n\n" % e)
+        return 1
 
     screen = urwid.raw_display.Screen()
     display = JMESPathDisplay(input_json)
     display.main(screen=screen)
+    display.display_saved_expressions()
+    return 0
 
 
 if __name__ == '__main__':
-    main()
+    sys.exit(main())
